@@ -1,12 +1,18 @@
+use log::debug;
+use maplit::hashmap;
+use serde_json::{json, Value};
+
 use pact_models::provider_states::ProviderState;
 use pact_models::sync_interaction::RequestResponseInteraction;
 use pact_models::v4::synch_http::SynchronousHttp;
 
 use super::request_builder::RequestBuilder;
 use super::response_builder::ResponseBuilder;
+use std::collections::HashMap;
 
 /// Builder for `Interaction` objects. Normally created via
 /// `PactBuilder::interaction`.
+#[derive(Clone, Debug)]
 pub struct InteractionBuilder {
     description: String,
     provider_states: Vec<ProviderState>,
@@ -18,12 +24,15 @@ pub struct InteractionBuilder {
 
     /// A builder for this interaction's `Response`.
     pub response: ResponseBuilder,
+    /// The interaction type (as stored in the plugin catalogue)
+    pub interaction_type: String,
 }
 
 impl InteractionBuilder {
   /// Create a new interaction.
-  pub fn new<D: Into<String>>(description: D) -> Self {
+  pub fn new<D: Into<String>>(description: D, interaction_type: D) -> Self {
     InteractionBuilder {
+      interaction_type: interaction_type.into(),
       description: description.into(),
       provider_states: vec![],
       comments: vec![],
@@ -69,15 +78,41 @@ impl InteractionBuilder {
 
   /// The interaction we've built (in V4 format).
   pub fn build_v4(&self) -> SynchronousHttp {
+    debug!("Building V4 HTTP interaction: {:?}", self);
+
     SynchronousHttp {
       id: None,
       key: None,
       description: self.description.clone(),
       provider_states: self.provider_states.clone(),
-      request: self.request.build().as_v4_request(),
-      response: self.response.build().as_v4_response(),
-      comments: Default::default(),
-      pending: false
+      request: self.request.build_v4(),
+      response: self.response.build_v4(),
+      comments: hashmap!{
+        "text".to_string() => json!(self.comments),
+        "testname".to_string() => json!(self.test_name)
+      },
+      pending: false,
+      plugin_config: self.plugin_config(),
+      interaction_markup: self.request.interaction_markup().merge(self.response.interaction_markup()),
+      transport: None
     }
   }
+
+  /// Any plugin configuration returned from plugins
+  pub fn plugin_config(&self) -> HashMap<String, HashMap<String, Value>> {
+    let mut config = hashmap!{};
+    let request_config = self.request.plugin_config();
+    if !request_config.is_empty() {
+      for (key, value) in request_config {
+        config.insert(key.clone(), value.interaction_configuration.clone());
+      }
+    }
+    let response_config = self.response.plugin_config();
+    if !response_config.is_empty() {
+      for (key, value) in response_config {
+        config.insert(key.clone(), value.interaction_configuration.clone());
+      }
+    }
+    config
+  }
 }
